@@ -11,15 +11,24 @@ namespace Data {
 
 struct StarsRating {
 	int level = 0;
-	int levelStars = 0;
-	int currentStars = 0;
+	int stars = 0;
+	int thisLevelStars = 0;
 	int nextLevelStars = 0;
 
 	explicit operator bool() const {
-		return level != 0 || levelStars != 0;
+		return level != 0 || thisLevelStars != 0;
 	}
 
 	friend inline bool operator==(StarsRating, StarsRating) = default;
 };
 
+struct StarsRatingPending {
+	StarsRating value;
+	TimeId date = 0;
+
+	explicit operator bool() const {
+		return value && date;
+	}
+};
+
 } // namespace Data
